#!/usr/bin/env node

const Valgrind = require('../lib/valgrind.js');
const JUnit = require('../lib/junit.js');
const Sanitizers = require('../lib/sanitizers.js');
const OrbisCoreView = require('../lib/orbis.js');
const DrMemory = require('../lib/drmemory.js');
const server = require('../lib/server.js');
const fs = require('fs');
const mktemp = require('mktemp');
const axios = require('axios');
const ProgressBar = require('progress');

const exclusion_string = [
  'leaks',
  'errors'
];

function toParser(format, path, options) {
  if (!options)
    options = {};

  if (format === 'valgrind') {
    return new Valgrind({ file: path, exclude: options.exclude });
  } else if (format === 'junit') {
    return new JUnit({ file: path, exclude: options.exclude });
  } else if (format === 'sanitizers') {
    return new Sanitizers({ file: path, exclude: options.exclude });
  } else if (format === 'drmemory') {
    return new DrMemory({ file: path, exclude: options.exclude });
  } else if (format === 'orbiscoreview') {
    return new OrbisCoreView({ file: path, exclude: options.exclude });
  }

  return null;
}

function send(argv, result) {
  var bar = new ProgressBar('[:current/:total] Sending... (:elapsed)', {
    renderThrottle: 100,
    total: result.length
  });
  var uuid = {};

  for (var i = 0; i < result.length; i++) {
    (function(r) {
      axios.post(argv.url, result[i])
        .then((response) => {
          if (!uuid[result[r].uuid]) {
            uuid[result[r].uuid] = true;
            bar.interrupt("  + " + result[r].uuid);
          }
          bar.tick();
          if (bar.complete) {
            console.log('Generated and sent ' + result.length + ' reports.');
          }
        })
        .catch((error) => {
          bar.tick();
          console.error('Error: ' + error);
        });
    })(i);
  }

  return;
}

function put(argv) {
  var data = [];
  var count = argv.file.length;
  var attributes = {};

  if (argv.attribute) {
    if (typeof argv.attribute === "string")
       argv.attribute = [argv.attribute];

    for (var i = 0; i < argv.attribute.length; i++) {
      var tokens = argv.attribute[i].split("=");

      attributes[tokens[0]] = tokens[1];
    }
  }

  var bar = new ProgressBar('[:current/:total] Parsing... (:elapsed)', {
    renderThrottle: 100,
    total: argv.file.length,
  });

  for (var i = 0; i < argv.file.length; i++) {
    var parser = toParser(argv.format, argv.file[i], {
      exclude: argv.exclude
    });

    if (parser == null) {
      count--;
      bar.tick();
      console.error('Unknown format: ' + argv.format);
      return;
    }

    parser.parse({ attributes: attributes }, (error, reports) => {
      if (error) {
        bar.tick();
        count--;
        if (count === 0)
          return send(argv, data);
      }

      var result = reports.serialize();
      data = data.concat(result);

      bar.tick();
      count--;
      if (count === 0)
        return send(argv, data);
    });
  }

  return;
}

function parse(argv) {
  var parser = toParser(argv.format, argv.file);
  var attributes = {};

  if (argv.attribute) {
    if (typeof argv.attribute === "string")
       argv.attribute = [argv.attribute];

    for (var i = 0; i < argv.attribute.length; i++) {
      var tokens = argv.attribute[i].split("=");

      attributes[tokens[0]] = tokens[1];
    }
  }

  parser.parse({ attributes : attributes }, (error, reports) => {
    if (error) {
      console.error(error + '');
      return;
    }

    var result = reports.serialize();

    console.error(result.length + ' errors were successfully extracted.');

    if (argv['output-directory']) {
      var od = argv['output-directory'];

      try {
        fs.mkdirSync(od);
      } catch (error) {

      }

      for (var i = 0; i < result.length; i++) {
        var path = mktemp.createFileSync(od + '/' + i + '-XXXXX.json');

        fs.writeFileSync(path, JSON.stringify(result[i], null, 2));
      }
    } else {
      console.log(JSON.stringify({ reports : result }, null, 2));
    }
  });
}

require('yargs')
  .scriptName("mela")
  .usage('$0 <command> [arguments]')
  .command('parse <file>', 'Parse a memory error log', (argv) => {
    argv.positional('file', {
      type: 'string',
      describe: 'An input file to parse'
    })
    .option('output-directory', {
      type: 'string',
      describe: 'An output directory to store reports in.'
    })
    .option('format', {
      type: 'string',
      describe: 'An input mode to parse'
    })
    .option('attribute', {
      type: 'string',
      describe: 'Attributes to add to errors'
    })
    .demandOption(['format'])
  }, parse)
  .command('put <file..>', 'Push memory error log to Backtrace', (argv) => {
    argv.positional('file', {
      type: 'string',
      describe: 'An input file to parse'
    })
    .array('file')
    .option('url', {
      type: 'string',
      describe: 'The URL to your Backtrace storage end-point'
    })
    .option('format', {
      type: 'string',
      describe: 'An input mode to parse'
    })
    .option('attribute', {
      type: 'string',
      describe: 'Attributes to add to errors'
    })
    .array('attribute')
    .option('exclude', {
      type: 'string',
      describe: 'Exclude any of leaks or errors'
    })
    .array('exclude')
    .demandOption(['format','url'])
  }, put)
  .command('server', 'Start a forwarding server', (argv) => {}, server)
  .help()
  .argv


//-- vim:ts=2:et:sw=2:st=2
