const fs = require('fs');

const Report = require('../lib/report.js');

function extractError(root, lines, i) {
  /* First attempt to extract the type of error. */
  var is_error = lines[i].match(/Error #\d+: ([A-Z ]+) .*:.*0x(.+)-/);
  var leaked_bytes;

  if (!is_error) {
    var is_leak = lines[i].match(/Error #\d+: ([A-Z ]+) (\d+) .* bytes/);

    if (!is_leak) {
      var is_other = lines[i].match(/Error #\d+: ([A-Z ]+) .* 0x(.+)/);
      if (!is_other) {
        console.error('Ignored: ' + lines[i]);
        return i + 1;
      }
      is_error = is_other;
    } else {
      leaked_bytes = is_leak[2];
      is_error = is_leak;
    }
  }

  var begin = i;
  var report = new Report(root);
  var classifier = is_error[1];
  var address = parseInt(is_error[2], 16);

  if (leaked_bytes >= 0) report.set('attribute', 'leaked.bytes', leaked_bytes);

  /* Translate the classifier to lower-case and replace spaces with "-". */
  classifier = classifier.toLowerCase().replace(/ /g, '-');

  report
    .set('attribute', 'fault.address', address)
    .add('classifiers', classifier);

  /* Now, subsequent lines should be a callstack. */
  var frames = [];
  var faulting_frame;

  for (i++; i < lines.length; i++) {
    var is_line = lines[i].match(/# \d+ (.*)\[(.*):(\d+)\]/);
    var frame = {};

    if (is_line) {
      var j;

      frame.file = is_line[2];
      frame.line = is_line[3];

      for (j = is_line[1].length - 1; j >= 0; j--) {
        if (is_line[1][j] !== ' ') break;
      }

      frame['function'] = is_line[1].substring(0, j + 1);

      frames.push(frame);
      if (!faulting_frame) faulting_frame = frame;

      continue;
    }

    /*
     * If no symbol information is available, the callstack will look
     * different.
     */
    var is_frame = lines[i].match(/# \d+ (.*) \+0x.* \(0x(.*) <(.*)+(.*)>\)/);
    if (is_frame) {
      for (var j = is_frame[1].length - 1; j >= 0; j--) {
        if (is_frame[1][j] !== ' ') break;
      }

      frame['function'] = is_frame[1].substring(0, j + 1);
      frame.registers = {};
      frame.registers.pc = '0x' + is_frame[2];
      frame.object = is_frame[3].split('+')[0];

      frames.push(frame);
      if (!faulting_frame) faulting_frame = frame;

      continue;
    }

    break;
  }

  /*
   * We should now be in the notes section. From here, we will extract
   * the thread identifier as well as a timestamp.
   */
  var pid;
  var skip = false;

  for (; i < lines.length; i++) {
    if (lines[i].indexOf('Note: ') === 0) {
      var is_time = lines[i].match(/Note: @\d+:\d+:\d+.\d+ in thread (.*)/);

      if (is_time) {
        pid = is_time[1];

        /* Extract the thread identifier. */
        report.set('attribute', 'pid', pid);
        break;
      }

      continue;
    }

    /* We went ahead too much and hit the next error. */
    if (lines[i].indexOf('Error #') === 0) {
      i--;
      skip = true;
      break;
    }
  }

  if (!faulting_frame) return i;

  /* Finally, scan forward to the next error, if one exists. */
  for (; skip === false && i < lines.length; i++) {
    if (lines[i] === '') break;

    if (lines[i].indexOf('Error #') === 0) {
      i--;
      break;
    }
  }

  /* Construct a text buffer for the faulting frame. */
  var thread;
  if (pid) {
    thread = report.thread(pid);
  } else {
    thread = report.thread('Thread');
  }

  faulting_frame = null;
  for (var j = 0; j < frames.length; j++) {
    var frame;

    frame = thread.frame(frames[j]);
    if (!faulting_frame) faulting_frame = frame;
  }

  if (faulting_frame) {
    var buffer = '';
    for (var j = begin; j < i; j++) buffer += lines[j] + '\n';

    faulting_frame.text = buffer;
  }

  return i;
}

function extractMetadata(report, line) {
  var is_version = line.match(/version .* build .* built on .*/);

  if (is_version) {
    var version = line;
    report.set('annotation', { Preamble: line });
  }

  var is_pid = line.match(/results for pid (.+): "(.+)"/);
  if (is_pid) {
    report.set('attribute', 'pid', is_pid[1]);

    var application = is_pid[2]; /* XXX: basename. */
    report.set('attribute', 'application', application);
  }

  return;
}

class drmemory {
  constructor(options) {
    this.options = options;
    this.file = options.file;
    this.data = options.data;
    this.exclude = options.exclude;
    if (!this.exclude) this.exclude = [];
  }

  parseData(options, cb) {
    var self = this;
    var lines = self.data.split('\n');
    var report = new Report();
    var checksum = report.checksum(self.data);

    report.set('attribute', 'tool', 'DrMemory').add('classifiers', 'DrMemory');

    /*
     * The general parsing strategy:
     *   - Extract an error at any line starting with "Error #<digits>:".
     *   - Metadata is extracted from lines beginning with "Dr. Memory".
     */
    for (var i = 0; i < lines.length; i++) {
      if (lines[i].indexOf('Dr. Memory ') === 0) {
        extractMetadata(report, lines[i]);
        continue;
      }

      if (lines[i].indexOf('Error #') === 0) {
        i = extractError(report, lines, i);
        continue;
      }
    }

    cb(null, report);
  }

  parse(options, cb) {
    var self = this;

    if (self.data) {
      return self.parseData(options, cb);
    } else if (self.file) {
      fs.readFile(this.file, 'utf8', function (e, d) {
        if (e) {
          return cb(e, null);
        }

        self.data = d;
        return self.parseData(options, cb);
      });
    } else {
      return cb('no data was supplied');
    }
  }
}

module.exports = drmemory;

//-- vim:ts=2:et:sw=2:st=2
